<?php
// inc_fetchrequest.php
// Holen und aufbereiten der API Daten

function httpGet($param) {
    global $wpps;

    // Check if global $wpps is defined and has required settings
    if (!isset($wpps) || !isset($wpps->settings['shopid'], $wpps->settings['apiurl'])) {
        return ['response' => false, 'info' => null];
    }

    $param['ShopId'] = $wpps->settings['shopid'];
    $url_params = http_build_query($param);
    $full_url = $wpps->settings['apiurl'] . '?' . $url_params;

    $response = false;
    $info = null;

    if (function_exists('curl_init')) {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $wpps->settings['apiurl']);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $param);
        curl_setopt($ch, CURLOPT_USERAGENT, $_SERVER['HTTP_USER_AGENT'] ?? 'PHP-cURL');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);

        $response = curl_exec($ch);
        $info = $ch ? curl_getinfo($ch, CURLINFO_HTTP_CODE) : null;

        curl_close($ch); // Always close cURL handle
    } else {
        $wpps->raiseError('Dieses Plugin benötigt das Modul cURL, um die Texte von der Protected Shops Schnittstelle abzurufen.');
    }

    return ['response' => $response, 'info' => $info];
}

function getDocumentInfo() {
    global $wpps;

    // Ensure global $wpps is defined
    if (!isset($wpps)) {
        return [];
    }

    $http = httpGet(['Request' => 'GetDocumentInfo']);
    $data = []; // Initialize as an empty array to avoid issues with `false`

    if ($http['response'] !== false && $http['info'] === 200) {
        try {
            $xml = new SimpleXMLElement($http['response']);

            if (isset($xml->error[0])) {
                // API error message
                $wpps->raiseError('Die Schnittstelle meldet: ' . htmlspecialchars((string)$xml->error[0]->msg, ENT_QUOTES));
            } else {
                // Convert XML structure to array
                $i = 0;
                foreach ($xml->DocumentDate[0] as $document => $version) {
                    $data[$i]['id'] = $document;
                    $data[$i]['version'] = (string)$version;
                    $i++;
                }
            }
        } catch (Exception $e) {
            $wpps->raiseError(
                'Daten können momentan nicht abgerufen werden. Überprüfen Sie, ob der Dienst verfügbar ist und ob die eingegebene API-URL korrekt ist. HTTP-Status Code: ' . htmlspecialchars((string)$http['info'], ENT_QUOTES)
            );
        }
    } else {
        $wpps->raiseError(
            'Daten können momentan nicht abgerufen werden. Überprüfen Sie, ob der Dienst verfügbar ist und ob die eingegebene API-URL korrekt ist. HTTP-Status Code: ' . htmlspecialchars((string)$http['info'], ENT_QUOTES)
        );
    }

    return $data;
}

function getDocument($did, $format) {
    global $wpps;

    if (!isset($wpps)) {
        return false;
    }

    $http = httpGet(['Request' => 'GetDocument', 'Document' => $did, 'Format' => $format]);

    if ($http['response'] !== false) {
        try {
            $xml = new SimpleXMLElement($http['response'], LIBXML_NOCDATA);
            return $xml;
        } catch (Exception $e) {
            $wpps->raiseError('Fehler beim Verarbeiten der XML-Daten: ' . $e->getMessage());
            return false;
        }
    } else {
        $wpps->raiseError('Daten können momentan nicht abgerufen werden.');
        return false;
    }
}
?>
