<?php
// inc_global.php
// Bereitstellen der globalen Datenstruktur von wpps

class wpps {
    public $settings;
    public $silentmode;
    public $filesdir;
    public $plugins = [];
    public $forceupdate;
    public function __construct() {
        $this->settings = false;
        $this->silentmode = false;
        $this->filesdir = plugin_dir_path(__FILE__) . 'files';
        $this->plugins = [];
        $this->forceupdate = false;
    }

    public function setSilent() {
        $this->silentmode = true;
    }

    public function addPlugin($name) {
        $this->plugins[0] = $name;
    }

    public function updateAll($maxchanges = 99) {
        if ($this->updateShortcodes($maxchanges)) {
            update_option('wpps_lastupdate', current_time('timestamp', 0));
        }
    }

    private function updateShortcodes($maxchanges) {
        global $wpps;

        require_once('inc_fetchrequest.php');

        $data = getDocumentInfo();

        $updatecount = 0;
        $newcount = 0;
        $nochangecount = 0;
        $failcount = 0;
        $changecount = 0;

        if (is_iterable($data)) {
            foreach ($data as $document) {
                if ($changecount >= $maxchanges) {
                    break;
                }

                $thisdoc = get_page_by_title($document['id'], OBJECT, 'wpps-shortcodes');

                if ($thisdoc !== null) {
                    $current_version = get_post_meta($thisdoc->ID, 'wpps-version', true);
                    $expected_version = $document['version'] . '/' . ($wpps->settings['format'] ?? '');

                    if ($current_version === $expected_version && !$this->forceupdate) {
                        $nochangecount++;
                    } else {
                        $doctext = $this->getDocText($document['id'], $wpps->settings['format']);
                        $dochash = $this->getDocHash($document['id'], $wpps->settings['format']);

                        if (md5($doctext) === $dochash) {
                            update_post_meta($thisdoc->ID, 'wpps-version', $expected_version);
                            update_post_meta($thisdoc->ID, 'wpps-content', $doctext);
                            do_action('wpps-update-document', $document);
                            $updatecount++;
                        } else {
                            $wpps->raiseError('Data integrity check failed for document ' . htmlspecialchars($document['id'], ENT_QUOTES));
                            $failcount++;
                        }
                        $changecount++;
                    }
                } else {
                    $newdoc = wp_insert_post([
                        'post_title' => $document['id'],
                        'post_status' => 'publish',
                        'post_type' => 'wpps-shortcodes',
                    ]);

                    $doctext = $this->getDocText($document['id'], $wpps->settings['format']);
                    $dochash = $this->getDocHash($document['id'], $wpps->settings['format']);

                    if (md5($doctext) === $dochash) {
                        add_post_meta($newdoc, 'wpps-version', $document['version'] . '/' . $wpps->settings['format'], true);
                        add_post_meta($newdoc, 'wpps-content', $doctext, true);
                        add_post_meta($newdoc, 'wpps-aktiv', '1', true);
                        add_post_meta($newdoc, 'wpps-status', '0', true);
                        add_post_meta($newdoc, 'wpps-lastpostid', '0', true);
                        do_action('wpps-new-document', $document);
                        $newcount++;
                    } else {
                        $wpps->raiseError('Data integrity check failed for document ' . htmlspecialchars($document['id'], ENT_QUOTES));
                        $failcount++;
                    }
                    $changecount++;
                }
            }
        }

        if (($updatecount + $nochangecount + $newcount + $failcount) > 0) {
            $this->raiseSuccess('Update complete. Updated: ' . $updatecount . ', New: ' . $newcount . ', No Changes: ' . $nochangecount . ', Errors: ' . $failcount);
        }

        return ($updatecount + $nochangecount + $newcount + $failcount) === count($data ?? []);
    }

    public function getDocText($did, $format) {
        $data = getDocument($did, $format);

        if ($data === false) {
            return false;
        }

        if ($format === 'Text') {
            return normalizeText((string) $data->Document);
        }

        return (string) $data->Document;
    }

    public function getDocHash($did, $format) {
        $data = getDocument($did, $format);
        return $data ? (string) $data->MD5 : false;
    }

    public function getShortcodeText($doc, $plaintext = false, $striptags = false) {
        $thisdoc = get_page_by_title($doc, OBJECT, 'wpps-shortcodes');
        if (!$thisdoc) {
            return '';
        }

        $output = '';
        if (get_post_meta($thisdoc->ID, 'wpps-aktiv', true) === '1') {
            update_post_meta($thisdoc->ID, 'wpps-status', '1');
            update_post_meta($thisdoc->ID, 'wpps-lastpostid', get_the_ID());

            if (!$plaintext) {
                $output = get_post_meta($thisdoc->ID, 'wpps-content', true);

                if ($striptags) {
                    $output = preg_replace('#<br\s*/?>#i', "\n", $output);
                    $output = str_replace('&nbsp;', "\n", $output);
                    $output = strip_tags($output);
                }

                $output .= '<style>/* ProtectedShops */ div.ps_document { max-width: 60em; } div.ps_document td { vertical-align: top; } div.ps_document td.text { text-align: justify; } /* END ProtectedShops */</style>';
            } else {
                $output = get_post_meta($thisdoc->ID, 'wpps-plaintext', true);
            }
        }

        return $output;
    }

    public function installPlugin() {
        update_option('wpps_updatetype', 'auto');
        update_option('wpps_interval', '24');
        update_option('wpps_apiurl', 'https://www.protectedshops.de/api/');
        update_option('wpps_format', 'Html');
    }

    public function checkUsage($saved_post_id) {
        $saved_post = get_post($saved_post_id);
        if (!$saved_post || !property_exists($saved_post, 'post_content')) {
            return;
        }

        preg_match_all('/\[wpps doc="(.*?)".*\]/i', $saved_post->post_content, $atts);
        $atts = $atts[1] ?? [];

        $args = [
            'post_type' => 'wpps-shortcodes',
            'meta_query' => [
                [
                    'key' => 'wpps-lastpostid',
                    'value' => $saved_post_id,
                ],
            ],
        ];

        $myposts = get_posts($args);

        foreach ($myposts as $post) {
            $is_used = in_array($post->post_title, $atts, true);
            update_post_meta($post->ID, 'wpps-status', $is_used ? '1' : '0');
        }

        foreach ($atts as $sinstance) {
            $used_shortcode = get_page_by_title($sinstance, OBJECT, 'wpps-shortcodes');
            if ($used_shortcode) {
                update_post_meta($used_shortcode->ID, 'wpps-status', '1');
                update_post_meta($used_shortcode->ID, 'wpps-lastpostid', $saved_post_id);
            }
        }
    }

    public function raiseSuccess($msg) {
        if (!$this->silentmode && is_admin()) {
            echo '<div class="updated"><p><strong>' . htmlspecialchars($msg, ENT_QUOTES) . '</strong></p></div>';
        }
    }

    public function raiseWarning($msg) {
        if (!$this->silentmode && is_admin()) {
            echo '<div class="update-nag"><p><strong>' . htmlspecialchars($msg, ENT_QUOTES) . '</strong></p></div>';
        }
    }

    public function raiseError($msg) {
        if (!$this->silentmode && is_admin()) {
            echo '<div class="error"><p><strong>' . htmlspecialchars($msg, ENT_QUOTES) . '</strong></p></div>';
        }
    }
}

function wpps_init() {
    register_post_type('wpps-shortcodes', [
        'labels' => [
            'name' => __('Shortcodes'),
            'singular_name' => __('Shortcode'),
        ],
        'public' => true,
        'has_archive' => false,
        'rewrite' => ['slug' => 'wpps-shortcodes'],
        'supports' => ['title', 'custom-fields'],
    ]);
}

add_action('init', 'wpps_init');
?>
